
library ieee;
library unisim ;

use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned."+";
use ieee.std_logic_unsigned."-";
use ieee.std_logic_unsigned.conv_integer;
use unisim.vcomponents.all ;


entity prio_dec is
    generic ( 
        NUM_READ_ONLY_ENTRIES: Natural;
        REF_LEFT            : Natural;
        SP_LEFT             : Natural;
        NUM_ENTRIES         : Natural );
    port (
        out_address_cat     : in std_logic_vector(
                    ((SP_LEFT + 1) * NUM_ENTRIES) - 1 downto 0);
        match_out_address   : out std_logic_vector(SP_LEFT downto 0);
        match_ro            : out std_logic;
        match_rw            : out std_logic;
        row_activation      : in std_logic_vector(0 to NUM_ENTRIES-1);
        row_match_1         : in std_logic_vector(0 to NUM_ENTRIES-1);
        row_match_2         : in std_logic_vector(0 to NUM_ENTRIES-1));
end entity prio_dec;

architecture basic of prio_dec is


    subtype OUT_ADDRESS_ENTRY is std_logic_vector ( SP_LEFT downto 0 ) ;
    type OUT_ADDRESS_ARRAY is array (0 to NUM_ENTRIES-2) of OUT_ADDRESS_ENTRY;
    signal out_address      : OUT_ADDRESS_ARRAY ;

    signal match_ref        : std_logic_vector ( REF_LEFT downto 0 ) ;
begin

--  process ( row_match_1, row_match_2, row_activation ) is
--      variable A      : std_logic_vector(NUM_ENTRIES downto 1);
--      variable B      : std_logic_vector(NUM_ENTRIES downto 1);
--      variable PD     : std_logic_vector(NUM_ENTRIES downto 1);
--  begin
--      match_ref <= (others => '0') ;
--      for output_bit in 0 to REF_LEFT
--      loop
--          A := (others => '0');
--          B := (others => '1');
--          B(NUM_ENTRIES) := '0';
--          for row in 1 to NUM_ENTRIES - 1
--          loop
--              if (( row / ( 2 ** output_bit ) mod 2 ) = 1 )
--              then
--                  A(row) := row_match_1(row) and row_match_2(row)
--                          and row_activation(row);
--              else
--                  B(row) := not (row_match_1(row) and row_match_2(row)
--                          and row_activation(row));
--              end if; 
--          end loop;
--          PD := A + B;
--          match_ref(output_bit) <= PD(NUM_ENTRIES);
--      end loop;
--  end process;

    process ( row_match_1, row_match_2, row_activation ) is
    begin
        match_ref <= (others => '0') ;
        for row in 1 to NUM_ENTRIES - 1
        loop
            if (( row_match_1 (row) and row_match_2 (row)
            and row_activation (row) ) = '1')
            then
                match_ref <= conv_std_logic_vector(row, REF_LEFT + 1);
            end if;
        end loop;
    end process;

    process ( match_ref , out_address_cat ) is
    begin
        match_ro <= '0';
        match_rw <= '0';
        match_out_address <= out_address(0);
        if (match_ref /= conv_std_logic_vector(0, REF_LEFT + 1))
        then
            if (match_ref <= conv_std_logic_vector(
                            NUM_READ_ONLY_ENTRIES, REF_LEFT + 1))
            then
                match_ro <= '1';
            else
                match_rw <= '1';
            end if;
            match_out_address <= out_address(conv_integer(match_ref) - 1);
        end if;
    end process;

    process ( out_address_cat ) is
        variable x : Natural;
    begin
        for i in 1 to ( NUM_ENTRIES - 1 )
        loop
            x := i * (SP_LEFT + 1);
            out_address(i - 1) <= out_address_cat(x + SP_LEFT downto x);
        end loop ;
    end process ;

end architecture basic;

